package io.gitlab.jfronny.commons.serialize.databind.impl.adapter;

import io.gitlab.jfronny.commons.serialize.databind.*;
import io.gitlab.jfronny.commons.serialize.databind.api.*;

import java.util.Optional;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;

public class SerializationAdapterTypeAdapterFactory implements TypeAdapterFactory {
    private final ConcurrentMap<Class<?>, Optional<TypeAdapterFactory>> adapterFactoryMap = new ConcurrentHashMap<>();

    @Override
    public <T> TypeAdapter<T> create(ObjectMapper mapper, TypeToken<T> type) {
        SerializeWithAdapter adapter = type.getRawType().getAnnotation(SerializeWithAdapter.class);
        if (adapter == null) {
            return null;
        }
        Optional<TypeAdapter<T>> result;
        if (TypeAdapter.class.isAssignableFrom(adapter.adapter())) {
            result = TypeUtils.instantiate(adapter.adapter());
        } else if (TypeAdapterFactory.class.isAssignableFrom(adapter.adapter())) {
            result = adapterFactoryMap.computeIfAbsent(type.getRawType(), TypeUtils::instantiate)
                    .map(factory -> factory.create(mapper, type));
        } else {
            result = Optional.empty();
        }
        return result.map(instance -> adapter.nullSafe() ? instance.nullSafe() : instance).orElse(null);
    }
}
