package io.gitlab.jfronny.commons.concurrent;

import io.gitlab.jfronny.commons.throwable.Assume;
import org.jetbrains.annotations.ApiStatus;

public interface WithScopedValue<T> {
    @ApiStatus.Internal ScopedValue<T> getAttached();
    @ApiStatus.Internal T self();

    default <TEx1 extends Throwable, TEx2 extends Throwable> void withContext(Action<TEx1, TEx2> action) throws TEx1, TEx2 {
        Assume.<TEx1>reintroduce();
        Assume.<TEx2>reintroduce(() -> ScopedValue.runWhere(getAttached(), self(), Assume.isSafe(action::run)));
    }
    default <T, TEx1 extends Throwable, TEx2 extends Throwable> T withContext(Returnable<T, TEx1, TEx2> action) throws TEx1, TEx2 {
        Assume.<TEx1>reintroduce();
        return Assume.<T, TEx2>reintroduce(() -> ScopedValue.getWhere(getAttached(), self(), Assume.isSafe(action::run)));
    }

    interface Returnable<T, TEx1 extends Throwable, TEx2 extends Throwable> {
        T run() throws TEx1, TEx2;
    }

    interface Action<TEx1 extends Throwable, TEx2 extends Throwable> {
        void run() throws TEx1, TEx2;
    }
}
